@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2019  Winfried Bruns, Bogdan Ichim, Christof Soeger
+ * Copyright (C) 2007-2021  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -92,9 +92,9 @@ struct CONVEXHULLDATA {
 template <typename Integer>
 struct SHORTSIMPLEX {
     vector<key_t> key;      // full key of simplex
-    Integer height;         // height of last vertex over opposite facet
+    Integer height;         // height of last vertex over opposite facet, used in Full_Cone
     Integer vol;            // volume if computed, 0 else
-    Integer mult;           // used for renf_elem_class
+    Integer mult;           // used for renf_elem_class in Full_Cone
     vector<bool> Excluded;  // for disjoint decomposition of cone
                             // true in position i indictate sthat the facet
                             // opposite of generator i must be excluded
@@ -258,6 +258,7 @@ class Cone {
     //---------------------------------------------------------------------------
 
     ~Cone();
+    void delete_aux_cones();
 
     //---------------------------------------------------------------------------
     //                          give additional data
@@ -274,9 +275,8 @@ class Cone {
     /* We allow the change of the cone by additional inequalities or generators
      * after the first computation for "dynamical" applications, in which
      * thecone is canged depending on previous computation results.
-     *
-     * Only one additional matrix can be added at a time. Nevertheless it is useful
-     * to go through the map.
+     * 
+     * If you want to add more than one type, use the map version.
      */
 
     void modifyCone(const map<InputType, vector<vector<Integer> > >& add_multi_input);
@@ -355,10 +355,6 @@ class Cone {
     Cone<Integer>& getSymmetrizedCone() const;
     Cone<Integer>& getProjectCone() const;
 
-    const Matrix<Integer>& getGeneratorsMatrix();
-    const vector<vector<Integer> >& getGenerators();
-    size_t getNrGenerators();
-
     const Matrix<Integer>& getExtremeRaysMatrix();
     const vector<vector<Integer> >& getExtremeRays();
     size_t getNrExtremeRays();
@@ -366,6 +362,10 @@ class Cone {
     const Matrix<nmz_float>& getVerticesFloatMatrix();
     const vector<vector<nmz_float> >& getVerticesFloat();
     size_t getNrVerticesFloat();
+    
+    const Matrix<nmz_float>& getExtremeRaysFloatMatrix();
+    const vector<vector<nmz_float> >& getExtremeRaysFloat();
+    size_t getNrExtremeRaysFloat();
 
     const Matrix<nmz_float>& getSuppHypsFloatMatrix();
     const vector<vector<nmz_float> >& getSuppHypsFloat();
@@ -403,6 +403,8 @@ class Cone {
 
     vector<Integer> getWitnessNotIntegrallyClosed();
     vector<Integer> getGeneratorOfInterior();
+    vector<Integer> getCoveringFace();
+    vector<Integer> getAxesScaling();
 
     const Matrix<Integer>& getHilbertBasisMatrix();
     const vector<vector<Integer> >& getHilbertBasis();
@@ -429,6 +431,10 @@ class Cone {
     const map<dynamic_bitset, int>& getFaceLattice();
     vector<size_t> getFVector();
     const vector<dynamic_bitset>& getIncidence();
+    
+    const map<dynamic_bitset, int>& getDualFaceLattice();
+    vector<size_t> getDualFVector();
+    const vector<dynamic_bitset>& getDualIncidence();
 
     // the actual grading is Grading/GradingDenom
     vector<Integer> getGrading();
@@ -454,6 +460,8 @@ class Cone {
     string getPolynomial() const;
 
     bool inequalities_present;
+    bool addition_generators_allowed;
+    bool addition_constraints_allowed;
 
     bool isPointed();
     bool isInhomogeneous();
@@ -461,6 +469,7 @@ class Cone {
     bool isDeg1HilbertBasis();
     bool isIntegrallyClosed();
     bool isGorenstein();
+    bool isEmptySemiOpen();
     bool isReesPrimary();
     bool isIntHullCone();
     Integer getReesPrimaryMultiplicity();
@@ -476,12 +485,13 @@ class Cone {
     // if no triangulation was computed so far they return false
     bool isTriangulationNested();
     bool isTriangulationPartial();
-    const vector<pair<vector<key_t>, Integer> >& getTriangulation();
- const vector<pair<vector<key_t>, Integer> >& getTriangulation(ConeProperty::Enum quality);
-    const vector<vector<bool> >& getOpenFacets();
+    const pair<vector<SHORTSIMPLEX<Integer> >, Matrix<Integer> >& getTriangulation();
+    const pair<vector<SHORTSIMPLEX<Integer> >, Matrix<Integer> >& getBasicTriangulation();
+    const pair<vector<SHORTSIMPLEX<Integer> >, Matrix<Integer> >& getTriangulation(ConeProperty::Enum quality);
+    const pair<vector<SHORTSIMPLEX<Integer> >, Matrix<Integer> >& getConeDecomposition();
     const vector<pair<vector<key_t>, long> >& getInclusionExclusionData();
-    const list<STANLEYDATA<Integer> >& getStanleyDec();
-    list<STANLEYDATA_int>& getStanleyDec_mutable();  // allows us to erase the StanleyDec
+    const pair<list<STANLEYDATA<Integer> >, Matrix<Integer> >& getStanleyDec();
+    pair<list<STANLEYDATA_int>, Matrix<Integer> >& getStanleyDec_mutable();  // allows us to erase the StanleyDec
                                                      // in order to save memeory for weighted Ehrhart
 
     bool get_verbose();
@@ -490,6 +500,7 @@ class Cone {
     IntegrationData& getIntData();
 
     void resetGrading(vector<Integer> lf);
+    void resetProjectionCoords(const vector<Integer>& lf);
 
     const Matrix<Integer>& getMatrixConePropertyMatrix(ConeProperty::Enum property);
     const vector<vector<Integer> >& getMatrixConeProperty(ConeProperty::Enum property);
@@ -514,6 +525,9 @@ class Cone {
     bool getBooleanConeProperty(ConeProperty::Enum property);
 
     nmz_float euclidean_corr_factor();
+    
+    vector<string> getRenfData();
+    renf_class* getRenf();
 
     //---------------------------------------------------------------------------
     //                          private part
@@ -543,12 +557,13 @@ class Cone {
     bool verbose;
     ConeProperties is_Computed;
     // Matrix<Integer> GeneratorsOfToricRing;
-    Matrix<Integer> OriginalMonoidGenerators;
+    Matrix<Integer> InputGenerators;
     Matrix<Integer> Generators;
-    Matrix<Integer> ReferenceGenerators;
+    // Matrix<Integer> ReferenceGenerators;
     Matrix<Integer> ExtremeRays;         // of the homogenized cone
     Matrix<Integer> ExtremeRaysRecCone;  // of the recession cone, = ExtremeRays in the homogeneous case
     Matrix<nmz_float> VerticesFloat;
+    Matrix<nmz_float> ExtremeRaysFloat;
     vector<bool> ExtremeRaysIndicator;
     Matrix<Integer> VerticesOfPolyhedron;
     Matrix<Integer> SupportHyperplanes;
@@ -558,12 +573,13 @@ class Cone {
     Integer TriangulationDetSum;
     bool triangulation_is_nested;
     bool triangulation_is_partial;
-    vector<pair<vector<key_t>, Integer> > Triangulation;
+    pair<vector<SHORTSIMPLEX<Integer> >, Matrix<Integer> > Triangulation; // the last computed triangulation
+    pair<vector<SHORTSIMPLEX<Integer> >, Matrix<Integer> > BasicTriangulation; // the basic triangulation
     vector<vector<bool> > OpenFacets;
     vector<bool> projection_coord_indicator;
     vector<pair<vector<key_t>, long> > InExData;
-    list<STANLEYDATA_int> StanleyDec;
-    list<STANLEYDATA<Integer> > StanleyDec_export;
+    pair<list<STANLEYDATA_int> , Matrix<Integer> >  BasicStanleyDec;
+    pair<list<STANLEYDATA<Integer> >, Matrix<Integer> > StanleyDec;
     mpq_class multiplicity;
     mpq_class volume;
     nmz_float euclidean_volume;
@@ -573,6 +589,8 @@ class Cone {
     mpq_class VirtualMultiplicity;
     vector<Integer> WitnessNotIntegrallyClosed;
     vector<Integer> GeneratorOfInterior;
+    vector<Integer> CoveringFace;
+    vector<Integer> AxesScaling;
     Matrix<Integer> HilbertBasis;
     Matrix<Integer> HilbertBasisRecCone;
     Matrix<Integer> BasisMaxSubspace;
@@ -590,25 +608,30 @@ class Cone {
     Integer unit_group_index;
     size_t number_lattice_points;
     vector<size_t> f_vector;
+    vector<size_t> dual_f_vector;
 
     vector<dynamic_bitset> Pair;        // for indicator vectors in project-and_lift
     vector<dynamic_bitset> ParaInPair;  // if polytope is a parallelotope
     bool check_parallelotope();
     bool is_parallelotope;
 
-    map<dynamic_bitset, int> FaceLattice;
+    map<dynamic_bitset, int> FaceLat;
+    map<dynamic_bitset, int> DualFaceLat;
     vector<dynamic_bitset> SuppHypInd;  // incidemnce vectors of the support hyperplanes
+    vector<dynamic_bitset> DualSuppHypInd;
 
     bool pointed;
     bool inhomogeneous;
     bool precomputed_extreme_rays;
     bool precomputed_support_hyperplanes;
+    bool empty_semiopen;
 
 
     bool input_automorphisms;
 
     bool polytope_in_input;
-    bool gorensetin;
+    bool rational_lattice_in_input;
+    bool inequalities_in_input;
 
     bool deg1_extreme_rays;
     bool deg1_hilbert_basis;
@@ -677,7 +700,7 @@ class Cone {
     void convert_lattice_generators_to_constraints(Matrix<Integer>& LatticeGenerators);
     void convert_equations_to_inequalties();
 
-    void check_gens_vs_reference();  // to make sure that newly computed generators agrre with the previously computed
+    // void check_gens_vs_reference();  // to make sure that newly computed generators agrre with the previously computed
 
     void setGrading(const vector<Integer>& lf);
     void setWeights();
@@ -694,6 +717,8 @@ class Cone {
     void make_Hilbert_series_from_pos_and_neg(const vector<num_t>& h_vec_pos, const vector<num_t>& h_vec_neg);
 
     void make_face_lattice(const ConeProperties& ToCompute);
+    void make_face_lattice_primal(const ConeProperties& ToCompute);
+    void make_face_lattice_dual(const ConeProperties& ToCompute);
     void compute_combinatorial_automorphisms(const ConeProperties& ToCompute);
     void compute_euclidean_automorphisms(const ConeProperties& ToCompute);
 
@@ -705,6 +730,12 @@ class Cone {
     
     void compute_refined_triangulation(ConeProperties& ToCompute);
 
+    template <typename IntegerFC>
+    void extract_automorphisms(AutomorphismGroup<IntegerFC>& AutomsComputed, const bool must_transform = false);
+    
+    void prepare_automorphisms();
+    void prepare_refined_triangulation(ConeProperties& ToCompute);
+
     template <typename IntegerColl>    
     void compute_unimodular_triangulation(ConeProperties& ToCompute);
     template <typename IntegerColl>
@@ -715,7 +746,7 @@ class Cone {
     void prepare_collection(ConeCollection<IntegerColl>& Coll);
     template <typename IntegerColl>
     void extract_data(ConeCollection<IntegerColl>& Coll);
-    void extract_data(ConeCollection<Integer>& Coll);
+    // void extract_data(ConeCollection<Integer>& Coll);
 
     /* only used by the constructors */
     void initialize();
@@ -723,9 +754,10 @@ class Cone {
 #ifdef NMZ_EXTENDED_TESTS
     void set_extended_tests(ConeProperties& ToCompute);
 #endif
-
-    template <typename IntegerFC>
+    
     void compute_full_cone(ConeProperties& ToCompute);
+    template <typename IntegerFC>
+    void compute_full_cone_inner(ConeProperties& ToCompute);
     
     void pass_to_pointed_quotient();
 
@@ -789,8 +821,9 @@ class Cone {
 
     void compute_vertices_float(ConeProperties& ToCompute);
     void compute_supp_hyps_float(ConeProperties& ToCompute);
+    void compute_extreme_rays_float(ConeProperties& ToCompute);
 
-    void make_StanleyDec_export();
+    void make_StanleyDec_export(const ConeProperties& ToCompute);
 
     void NotComputable(string message);  // throws NotComputableException if default_mode = false
 
@@ -832,7 +865,8 @@ class Cone {
                                                 Matrix<IntegerFC>& FC_Vectors,
                                                 const Matrix<Integer>& ConeVectors,
                                                 bool primal,
-                                                vector<key_t>& Key);
+                                                vector<key_t>& Key,
+                                                const bool must_transform);
 
     vector<vector<key_t> > extract_subsets(const vector<vector<key_t> >& FC_Subsets, size_t max_index, const vector<key_t>& Key);
 };
