@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2019  Winfried Bruns, Bogdan Ichim, Christof Soeger
+ * Copyright (C) 2007-2021  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -30,11 +30,11 @@
 #include <list>
 #include <iostream>
 #include <string>
-#include <math.h>
+#include <cmath>
 
 #include <libnormaliz/general.h>
 #include <libnormaliz/integer.h>
-#include <libnormaliz/convert.h>
+// #include <libnormaliz/convert.h>
 #include <libnormaliz/vector_operations.h>
 #include "libnormaliz/dynamic_bitset.h"
 // #include <libnormaliz/sublattice_representation.h>
@@ -500,7 +500,7 @@ class Matrix {
 
     // try to sort the rows in such a way that the extreme points of the polytope spanned by the rows come first
 
-    size_t extreme_points_first(const vector<Integer> norm = vector<Integer>(0));
+    size_t extreme_points_first(bool verbose, const vector<Integer> norm = vector<Integer>(0));
 
     // find an inner point in the cone spanned by the rows of the matrix
 
@@ -644,7 +644,7 @@ Matrix<Number> LLL_red(const Matrix<Number>& U, Matrix<Integer>& T, Matrix<Integ
             /* cout << "MMMMM " << i << " " << j << " " << M[i][j] << endl;
             cout << i << "---" << G[i];
             cout << j << "---" << G[j];*/
-            if (isnan(M[i][j])) {
+            if (std::isnan(M[i][j])) {
                 T = Tinv = Matrix<Integer>(U.nr_of_rows());
                 return U;
             }
@@ -759,7 +759,7 @@ void convert(Matrix<ToType>& to_mat, const Matrix<FromType>& from_mat) {
 // only set to false in this routine
 // if a set occurs more than once, only the last instance is recognized as maximal
 template <typename IncidenceVector>
-void maximal_subsets(const vector<IncidenceVector>& ind, vector<bool>& is_max_subset);
+void maximal_subsets(const vector<IncidenceVector>& ind, IncidenceVector& is_max_subset);
 
 }  // namespace libnormaliz
 
